#include <xsTypes.h>
#include <functionMap.h>

double fggrain(const char* element);

void tbgrain(const RealArray& energyArray, const RealArray& par,
        int spectrumNumber, RealArray& flux, RealArray& fluxErr, 
        const string& initString)
{
   //    Compute X-ray absorptivity of cold gas using the formalism given
   //    by Wilms, Allen, and McCray, ApJ, 2000, in press

   //    This version allows to vary the grain and h2 parameters

   //    The parameters are

   //     par[0]   hydrogen column (1E22/cm^2)
   //     par[1]   fraction of h in h2 (default: 0.2)
   //     par[2]   density of dust (default: 1 g/cm^3)
   //     par[3]   minimum thickness, mum (0.025)
   //     par[4]   maximum thickness, mum (0.25)
   //     par[5]   PL index with implicit minus sign (3.5)

   //     Depletion et al. are still at their default values
   
   // Translated from tbgrain.f 03/09 (CG)
   
   
   // Preset other values with their defaults
   
   const size_t NELE = 18;
   const char *celts[] = {"H ", "He", "C ", "N ", "O ", "Ne", "Na", 
              "Mg", "Al", "Si", "S ", "Cl", "Ar", "Ca", "Cr", "Fe",
              "Co", "Ni"};
   
   RealArray param(.0, 42);
   //    H column
         param[ 0]=par[0];
   //    helium      [1]
         param[ 1]=1.;
   //    carbon      [5]
         param[ 2]=1.;
   //    nitrogen    [6]
         param[ 3]=1.;
   //    oxygen      [7]
         param[ 4]=1.;
   //    neon        [9]
         param[ 5]=1.;
   //    sodium      [10]
         param[ 6]=1.;
   //    magnesium   [11]
         param[ 7]=1.;
   //    aluminium   [12]
         param[ 8]=1.;
   //    silicon     [13]
         param[9]=1.;
   //    sulphur     [15]
         param[10]=1.;
   //    chlorine    [16]
         param[11]=1.;
   //    argon       [17]
         param[12]=1.;
   //    calcium     [19]
         param[13]=1.;
   //    chromium    [23]
         param[14]=1.;
   //    iron        [25]
         param[15]=1.;
   //    cobalt      [26]
         param[16]=1.;
   //    nickel      [27]
         param[17]=1.;
   //    param[18]: fraction of hydrogen existing in molecular form
         param[18]=par[1];

   //    param[19]: density of dust in g/cm^3
         param[19]=par[2];

   //    if param[20] = param[21]: use single size grain
   //      param[20]: minimum thickness of dust (mum)
         param[20]=par[3];
   //      param[21]: maximum thickness of dust (mum)
         param[21]=par[4];
   //    param[22]: PL index for dust size distribution (implicit minus-sign!)
         param[22]=par[5];

   //    param[23..40]: depletion factor (ratio of GAS to total ISM abundance)
   for (size_t i=0; i<NELE; ++i)
      param[23+i] = fggrain(celts[i]);
      
   // Redshift
   param[41] = 0.0;
   
   // Compute the absorption
   tbvabs(energyArray, param, spectrumNumber, flux, fluxErr, initString);
   
   //     ... that's all, folks!
}
